<?php

/**
 * BuddyServ IRC Services
 * 
 * Generates GLine network bans, or retrieves
 * from database. Provides all GLine handling
 * capabilities
 * 
 * @name Gline
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Ircd
 * @version $Id: Class.gline.php 318 2009-11-14 04:17:22Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class gline
{
	/* public fields are for databased passed fields ONLY
	 * we have __get magic, so name non-db to protected */
	public $id;
	public $added;
	public $expires;
	public $oper;
	public $mask;
	public $reason;
	
	protected $regex;
	
	/* static for all gline lists */
	static $list = array();
	
	/**
	 * Grab all the glines from the database and
	 * add it to our local memory.
	 * 
	 * @return void
	 */
	static function populate()
	{
		global $db, $irc, $me;
		
		/* reset */
		self::$list = array();
		
		/* get from db and create each object for storing */
		$query = $db->query('SELECT * FROM `buddyserv_glines`');
		
		if ($db->num_rows($query) == 0)
			return;

		while ($each = $db->fetch_array($query))
		{
			$obj = new gline();
			$userhost = explode('@', $each['mask']);
			
			/* check for expired glines */
			if (($each['expires']  < current_time()) and ($each['expires'] > 0))
			{
				$db->query("DELETE FROM `buddyserv_glines` WHERE `id` = ".$each['id']." LIMIT 1");
				
				// code for irc ungline
				$irc->send(sprintf(UNKLINE, $me->operserv['nick'], $userhost[0], $userhost[1]));
				$irc->send(sprintf(OPERWALL, $me->operserv['nick'], 'Expiring akill: '.$each['mask'].' set by '.$each['oper']));
				continue;
			}
			
			/* gline still active, propriate it across the network */
			$timeleft = $each['expires'] - current_time();
			$irc->send(sprintf(KLINE, $me->operserv['nick'], $timeleft, $userhost[0], $userhost[1], $each['reason']));
				
				//$irc->send(sprintf(KLINE, $me->operserv['name'], 
			//	:OperServ KLINE * 123 * webchat2.mibbit.com :You are banz
		
		
		foreach ($each as $detail => $value)
		{
			$obj->{$detail} = $value;
		}
		
		$mask = strtolower($each['mask']);
		
		if (!($obj instanceof gline))
			die('woah, not a valid gline being added...');
			
		self::$list[$mask] = $obj;
	}
	}

	/**
	 * Check if a gline for the current mask already exists
	 * 
	 * @param string $mask
	 * @return bool
	 */
	static function exists($mask)
	{
		$mask = strtolower($mask);
		
		if (array_key_exists($mask, self::$list))
		{
			return true;
		}
		
		return false;
	}
	
	/**
	 * Delete a gline from our memory
	 * 
	 * @param string $mask
	 * @return bool
	 */
	static function del($mask)
	{
		$mask = strtolower($mask);
		
		/* Verify the key exists */
		if (self::exists($mask))
		{
			unset(self::$list[$mask]);
			return true;
		}
		
		/* We didnt find that mask in the gline list
		 * return false */
		return false;
	}	
		
	/**
	 * Get an instance of a gline object,
	 * or returns false if not exist.
	 * 
	 * @param string $mask
	 * @return mixed
	 */
	static function get($mask)
	{
		$mask = strtolower($mask);
		
		if (self::exists($mask))
		{
			$pointer = &self::$list[$mask];
			
			if ($pointer instanceof gline)
				return $pointer;
				
			return false;
		}
		
		return false;
	}
	
	/**
	 * Expire glines that are past due.
	 * 
	 * @return void
	 */
	static function run_expires()
	{
		global $irc, $me, $db;
		$templist = self::$list;
		
		foreach ($templist as $mask => $object)
		{
			if (!$mask)
				continue;
				
			/* check if expired */
			if (($object->expires < current_time() and ($object->expires > 0)))
			{
				/* we dont care if we succeed. it may have already been removed (outside script)*/
				$query = $db->query("DELETE FROM `buddyserv_glines` WHERE `mask` = '".$mask."' LIMIT 1");
				
				$userhost = explode('@', $object->mask);
				$irc->send(sprintf(UNKLINE, $me->operserv['nick'], $userhost[0], $userhost[1]));
				$irc->send(sprintf(OPERWALL, $me->operserv['nick'], 'Expiring akill: '.$object->mask.' set by '.$object->oper));
				
				unset(gline::$list[$mask]);
			}
		}
	}
				
			
	/**
	 * Create a new gline to add to the database
	 * 
	 * @param int $expires
	 * @param string $oper
	 * @param string $mask
	 * @param string $reason
	 * @return bool
	 */
	static function create($expires,$oper,$mask,$reason)
	{
		global $db;
		
		/* woah, someone hasnt been doing proper sanity checks
		 * you should ot call create without first checking if
		 * the shit already exists. */
		if (self::exists($mask))
		{
			die('gline exists');
			return false;
		}
			
		$gline = new gline();
		
		$gline->expires = $expires;
		$gline->oper = $oper;
		$gline->mask = strtolower($mask);
		$gline->reason = $reason;
		
		/* create pre */
		$gline->regex = str_replace('?', '\S{1}', $mask);
		$gline->regex = str_replace('*', '\S*', $gline->regex);

		$query = $db->query("INSERT INTO `buddyserv_glines` (`added`, `expires`, `oper`, `mask`, `reason`) VALUES ('".current_time()."', '{$expires}', '{$oper}', '{$mask}', '{$reason}')");
		
		if ($db->insert_id($query))
		{
			$mask = strtolower($mask);
			gline::$list[$mask] = $gline;
			return true;
		}
		
		return false;	
	}

	/**
	 *  Magic getter function 
	 */
	public function __get($what)
	{
		return (($this->{$what}) ? $this->{$what} : null);
	} 
	
	/* Magic setter function */
	public function __set($what, $value)
	{
		$this->{$what} = $value;
	}
	
	/* Umm, we dont really need to do anything to start, but... */
	public function __construct()
	{
		
	}
	
}
	