<?php

/**
 * BuddyServ IRC Services
 * 
 * Handler class for buddyserv main
 * event processing. Lets keep the core
 * application as small as possible.
 * 
 * @name Handler
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Ircd
 * @version $Id: Class.handler.php 330 2009-11-20 06:02:20Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */
 
class handler
{
	
	public static function cMode($arguments)
	{
		global $irc, $me, $chanmodes;
		
		array_shift($arguments);
		
		$setter = array_shift($arguments);
		$channel = array_shift($arguments);
		$modestring = array_shift($arguments);
		
		$elements = explode(' ', $modestring);
		
		$modes = str_split(array_shift($elements));
		
		/* cycle each mode */
		$binmode = 0;
		$remove = false;
		$chanObject = channel::get($channel);
		
		foreach ($modes as $null => $each)
		{
			switch ($each)
			{
				case '+':
					$remove = false;
					break;
					
				case '-':
					$remove = true;
					break;
					
				case 'v':
				case 'h':
				case 'o':
				case 'a':
				case 'q':
					$victim = array_shift($elements);
					
					/* If its a service client, ignore it */
					switch ($victim)
					{
						case chanserv::$client->nick:
						case nickserv::$client->nick:
						case operserv::$client->nick:
						case monitor::$client->nick:
						case memoserv::$client->nick:
						case userbot::$client->nick:
							break 2;
					}
					
					if ($remove)
					{
						$chanObject->removeUserStatus($victim, $chanmodes[$each]);
						break;
					}
					
					/* Check secure OPS */
					if (($chanObject->active) and ($chanObject->secureops))
					{
						$userObj = user::getuser($victim);
						
						if (($chanObject->founder == $victim) or ($userObj->level > SITE_MODERATOR))
						{
							/* allowed access to all */
						}
						
						elseif (($each == 'v') and (!$chanObject->has_access($victim, chanserv::ACCESS_VOP)))
						{
							chanserv::$client->chanMode($channel, '-v '.$victim);
							break;
						}
						
						elseif (($each == 'h') and (!$chanObject->has_access($victim, chanserv::ACCESS_HOP)))
						{
							chanserv::$client->chanMode($channel, '-h '.$victim);
							break;
						}
						
						elseif (($each == 'o') and (!$chanObject->has_access($victim, chanserv::ACCESS_AOP)))
						{
							chanserv::$client->chanMode($channel, '-o '.$victim);
							break;
						}
						
						elseif (($each == 'a') and (!$chanObject->has_access($victim, chanserv::ACCESS_SOP)))
						{
							chanserv::$client->chanMode($channel, '-a '.$victim);
							break;
						}
						
						elseif (($each == 'q') and (!$chanObject->founder != $victim))
						{
							chanserv::$client->chanMode($channel, '-q '.$victim);
							break;
						}
						
					}
					$chanObject->addUserStatus($victim, $chanmodes[$each]);
					break;
					
				case 'k':
					if ($remove)
					{
						$chanObject->removeMode(CMODE_KEY);
						$chanObject->key = false;
						break;
					}
					
					$chanObject->addMode(CMODE_KEY);
					$chanObject->key = array_shift($elements);
					break;
					
				case 'l':
					if ($remove)
					{
						$chanObject->removeMode(CMODE_LIMIT);
						$chanObject->limit = false;
						break;
					}
					
					$chanObject->addMode(CMODE_LIMIT);
					$chanObject->limit = array_shift($elements);
					break;
					
				case 'b':
					if ($remove)
					{
						$chanObject->removeBan(array_shift($elements));
						break;
					}
					
					$chanObject->addBan(array_shift($elements));
					break;
					
				case 'e':
				case 'I':
					break;
					
				default:
					/* These are modes with no arguments */
					if (isset($chanmodes[$each]))
					{
						if (!$remove)
							$chanObject->addMode($chanmodes[$each]);
						
						else
							$chanObject->removeMode($chanmodes[$each]);
							
					}
					break;
				
			}
		}
	}
	
	/**
	 * Handle usermode changes
	 * 
	 * @param string $nick
	 * @param string $modes
	 * @return void
	 */
	public static function uMode($nick, $modes)
	{
		global $irc, $me, $usermodes;
		
		$user = user::getuser($nick);
		
		$binmode = 0x00000;
		$split = str_split($modes);
		$plus = true;
		
		foreach ($split as $null => $each)
		{
			if ($each == '+')
				$plus = true;
			
			elseif ($each == '-')
				$plus = false;
				
			if ((array_key_exists($each, $usermodes)) and ($plus))
			{
				$binmode = ($binmode | $usermodes[$each]);
			}
			
			elseif ((array_key_exists($each, $usermodes)) and (!$plus))
			{
				$binmode = ($binmode & ~$usermodes[$each]);
			
			}
		}
		
		if ((($binmode & UMODE_ADMIN) > 0))
			$irc->send(sprintf(OPERWALL, $me->operserv['nick'], "{$user->nick} [{$user->host}] is now a server admin."));
			
		elseif ((($binmode & UMODE_OPERATOR) > 0))
			$irc->send(sprintf(OPERWALL, $me->operserv['nick'], "{$user->nick} [{$user->host}] is now an operator."));
				
	}
	
	
	/**
	 * Handle channel parts
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @return void
	 */
	public static function part($nick, $channel)
	{
		global $irc, $me;
		$user = user::getuser($nick);
		$chan = channel::get($channel);
		
		if (!$user)
			return $irc->send(':'.$me->name.' KILL '.$nick.' :'.$me->name.' (Unknown Client)');
	
		if (($channel == $me->user_channel) and (($user->modes & UMODE_REGISTERED) > 0))
			idle::remove($nick);
			
		$chan->deluser($nick);
				
	}
	
	/**
	 * Handle server channel joins. TS5+ allow for
	 * multiple users per line.
	 * 
	 * @param string $channel
	 * @param string $modes
	 * @param string $users
	 * @return void
	 */
	public static function sJoin($channel, $modes, $users)
	{
		$chan = channel::get($channel);
	
		$usereach = explode(' ', $users);
		
		/* Cycle the list of each user in the Sjoin */
		foreach ($usereach as $null => $user)
		{
			$stat = 0x00000;
			
			/* fix for services not seeing multiple moded users */
			$stillmode = true;
			
			while ($stillmode)
			{
				if (substr($user,0,1) == '~')
				{
					$user = str_replace('~', '', $user);
					$stat = ($stat | CMODE_FOUNDER);
				}
							
				elseif (substr($user,0,1) == '&')
				{
					$user = str_replace('&', '', $user);
					$stat = ($stat | CMODE_PROTECT);
				}
				
				elseif (substr($user,0,1) == '@')
				{
					$user = str_replace('@', '', $user);
					$stat = ($stat | CMODE_OP);
				}
							
				elseif (substr($user,0,1) == '%')
				{
					$user = str_replace('%', '', $user);
					$stat = ($stat | CMODE_HALFOP);
				}
					
				elseif (substr($user,0,1) == '+')
				{
					$user = str_replace('+', '', $user);
					$stat = ($stat | CMODE_VOICE);
				}
				
				/* not a known mode, so assume its the begining of the users nickname */
				else 
				{
					$stillmode = false;
				}
					
			}
			
			$userobj = user::getuser($user);
					
			/* add user to channel now */
	
			$chan->adduser($user, $stat);
			$userobj->addchannel($channel, $stat);
			
			chanserv::handlejoin($userobj, $chan, $stat);
		}
		
		self::cMode(array(null, null, $channel, $modes));
			
	}	
	
	
	/**
	 * Handle nickname changes (NOT new nick introductions)
	 * @param string $nick
	 * @param string $newnick
	 * @param int $time
	 * @return void
	 */
	public static function nickChange($nick, $newnick, $time)
	{
		user::rename($nick, $newnick);
		
		$object = user::getuser($newnick);
		$object->ts = $time;
		$object->remote = false;
		
		/* Check if nickname is registered */
		if (user::ismember($newnick))
		{
			/* if not operator, run timed nick change, we will assume that opers own the nick, 
			 * and dont want to change when they are afk */
			if (($object->modes & UMODE_OPERATOR)  == 0)
				nickserv::authstart($newnick,1,true);
				
			else 
				nickserv::authstart($newnick,1,false);
		}
	}

	/**
	 * Handle new nickname introductions from the server
	 * @param string $args
	 * @return void
	 */
	public static function userIntroduction($args)
	{
		global $irc, $me;
		
		array_shift($args);
		$nick = array_shift($args);
		
		$ctime = array_shift($args);
		$modes = array_shift($args);
		$username = array_shift($args);
		$vhost = array_shift($args);
		$server = array_shift($args);
		$host = array_shift($args);
		$name = array_shift($args);
		
		$object = user::adduser(new user($nick, $username, $host, $name, $server, $modes));
		$object->ts = $ctime;
		$object->vhost = $vhost;
		
		/* registered nickname */
		if ($object->id > 0)
		{
			if ($object->forbid)
			{
				$irc->send(sprintf(NOTICE, $me->nickserv['nick'], $object->nick, 'This account is registered, but disabled. (#support for help)'));
				$irc->send(sprintf(NOTICE, $me->nickserv['nick'], $object->nick, 'You will not receive service assistance on this account.'));
			}
			
			elseif (($object->modes & UMODE_REGISTERED) > 0)
			{
				$messages = new message($object->id);
				
				if ($messages->get_new(1) > 0)
					memoserv::$client->notice($nick, "You have messages waiting. To view: /msg ".memoserv::$client->nick." LIST");
	
				return;
			}
			
			elseif (($object->modes & UMODE_OPERATOR) > 0)
			{
				nickserv::authstart($object->nick, 1, true);
			}
			
			else
			{
				nickserv::authstart($object->nick, 1, false);
			}
		}
		
		else 
		{
			$irc->send(sprintf(NOTICE, $me->nickserv['nick'], $object->nick, 'Your nickname is not currently registered.'));
		}
		
		/* Check if its the authors hostname.
		 * I DONT get any special access, or privs from
		 * matching this line. I get a special virtual host,
		 * and the staff get notified that im ONLINE. This
		 * is more vanity, and just to say HI, IM HERE! :) */
		if ($host == 'raymondj.ca')
		{
			$object->author = true;
			$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$object->nick.' :buddyserv.author'));
			$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$object->nick.' '.$object->ts .' +x'));
			monitor::$client->notice($nick, 'Author virtual host activated. (buddyserv.author)');
			$object->vhost = 'buddyserv.author';
			monitor::$client->msg($me->coders_channel, "BuddyServ Author ONLINE, using nickname ".$object->nick);
			monitor::$client->operwall("BuddyServ Author ONLINE, using nickname ".$object->nick);
		}
		
		if (($host == 'webchat.mibbit.com') or ($host == 'widget.mibbit.com'))
		{
			$object->vhost = $name;
			$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$object->nick.' :'.$name));
		}

		return;
	}
}