<?php

/**
 * BuddyServ IRC Services
 * 
 * MemoServ internal to IRC message relay system
 * 
 * @name MemoServ
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.memoserv.php 318 2009-11-14 04:17:22Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class memoserv
{
	static $client;
	static $verbose   =true;
	static $repeat = true;
	static $enabled=true;
	
	/**
	 * Initialize the current service client.
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	static function initialize($nick, $username, $host, $name)
	{
		self::$client = new serviceclient();
		self::$client->initialize($nick, $username, $host, $name);
	}
	
	/**
	 * Magic getter
	 * 
	 * @param string $what
	 * @return mixed
	 */
	function __get($what)
	{
		if (self::$client->$what)
			return self::$client->$what;
			
	}
	
	/**
	 * Introduce the current client to
	 * the irc server.
	 * 
	 * @return void
	 */
	static function introduce()
	{
		self::$client->introduce();
	}
	
	/**
	 * Called to parse any messages to monitor
	 *
	 * @param string $nick
	 * @param string $arguments
	 * @return void
	 */
	static function parser($nick, $arguments)
	{
		$checkuser = user::getuser($nick);
		
		if ($checkuser->forbid)
			return self::$client->notice($nick, "Access denied. Your user account is disabled.");
			
		if (($checkuser->modes & UMODE_REGISTERED) == 0)
			return self::$client->notice($nick, "Access denied. Please login to NickServ to do this command.");
			
		if (strcasecmp($arguments[0], 'LIST') == 0)
			return self::listmsg($nick);
			
		if (strcasecmp($arguments[0], 'READ') == 0)
			return self::read($nick, $arguments[1]);
		
		if (strcasecmp($arguments[0], 'DELETE') == 0)
			return self::delete($nick, $arguments[1]);
			
		if (strcasecmp($arguments[0], 'SAVE') == 0)
			return self::save($nick, $arguments[1]);
			
		if (strcasecmp($arguments[0], 'SEND') == 0)
			return self::send($nick, $arguments);
		
		if (strcasecmp($arguments[0], 'HELP') == 0)
		{
			array_shift($arguments);
			
			if (count($arguments) == 0)
				return self::$client->help($nick);
			
			return self::$client->help($nick, $arguments);
		}
		
		return self::unknowncmd($nick, $arguments[0]);
	}
	
	/**
	 * Save a received message for later viewing
	 * 
	 * @param string $nick
	 * @param int $id
	 * @return void
	 */
	static function save($nick, $id)
	{
		$user = user::getuser($nick);
		
		if (!intval($id))
			return self::$client->msg($nick, "Invalid or missing message ID for DELETE.");
			
		$message = new message($user->id);
		
		if (!$message->fetch($id))
			return self::$client->msg($nick, "Message with ID \2".$id."\2 does not exist.");
			
		if ($message->owner <> $user->id)
			return self::$client->msg($nick, "That message does not belong to you.");
			
		if (!$message->save())
			return self::$client->msg($nick, "An internal error has been logged. Please try again later.");
			
		else 
			return self::$client->msg($nick, "Message has been saved.");
	}
	
	/**
	 * Send a new mail message to a user
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function send($nick, $arguments)
	{
		$user = user::getuser($nick);
		array_shift($arguments);
		
		$receiver = array_shift($arguments);
		
		$arguments = implode(' ', $arguments);
		
		/* seperate the subject from the body */
		preg_match('/^\'(.+)\' (.+)/', $arguments, $matches);
		
		/* first match is always the full string, truncate it */
		array_shift($matches);
		
		if (count($matches) <> 2)
			return self::$client->msg($nick, "Invalid syntax. Please consult 'HELP SEND' for more information.");
			
		$subject = array_shift($matches);
		$body = array_shift($matches);
		
		$sendto = user::sql_single($receiver);
		
		if (!$sendto)
			return self::$client->msg($nick, "Unable to send message to '".$receiver."' as that account doesnt exist.");
			 
		if ($sendto['enabled'] == 'no')
			return self::$client->msg($nick, "Unable to send message to '".$receiver."' as their account is disabled.");
			
		$recv_id = $sendto['id'];
		
		$message = new message($recv_id);
		$message->send($user->id, mysql_real_escape_string($subject), mysql_real_escape_string($body));
		
		return self::$client->msg($nick, "Your message has been sent.");
		
	}
		
		
	
	/**
	 * Delete a message from the system
	 *
	 * @param string $nick
	 * @param integer $id
	 * @return null
	 */
	static function delete($nick, $id)
	{
		$user = user::getuser($nick);
		
		if (!intval($id))
			return self::$client->msg($nick, "Invalid or missing message ID for DELETE.");
			
		$message = new message($user->id);
				
		if (!$message->fetch($id))
			return self::$client->msg($nick, "Message with ID \2".$id."\2 does not exist.");
			
		if ($message->owner <> $user->id)
			return self::$client->msg($nick, "That message does not belong to you.");
			
		/* Try to delete and check for failure */
		if (!$message->delete())
			return self::$client->msg($nick, "An internal error has been logged. Please try again later.");
			
		else 
			return self::$client->msg($nick, "Message deleted with subject '".$message->subject."'");
	}
	
	/**
	 * Method for when someone wants to read a message
	 *
	 * @param string $nick
	 * @param integer $id
	 * @return void
	 */
	static function read($nick, $id)
	{
		$user = user::getuser($nick);
		
		$message = new message($user->id);
		
		if (!intval($id))
			return self::$client->msg($nick, "Message ID is missing or invalid.");
			
		if ($message->fetch($id) == false)
			return self::$client->msg($nick, "No message with ID ".$id." found.");
		
		/* Ensure this message actually belongs to the requesting user. */
		if ($message->owner <> $user->id)
			return self::$client->msg($nick, "Hey! That message doesn't belong to you!");
			
		self::$client->msg($nick, "[".$message->id."] \2Reading\2:\0033 ".$message->subject."\003");
		self::$client->msg($nick, "\2Sent\2:\0037 ".$message->added." \003by\2 ".$message->sender_name);
		self::$client->msg($nick, "-----------------------------------------");
		
		/* if body is too long to send in one message string,break it up */
		$body = $message->body;
		$body = str_replace("\n", " --- ", $body);
		
		while (strlen($body) > 200)
		{
			$spliced = substr($body, 0, 200);
			$body = substr($body, 200, strlen($body));
			
			self::$client->msg($nick, $spliced);
		}
		
		/* if anything left in the body, send it now */
		if (strlen($body) > 0)
			self::$client->msg($nick, $body);
			
		self::$client->msg($nick, "\0034------END OF MESSAGE------\003");
	}
	
	/**
	 * Function for listing available messages
	 *
	 * @param string $nick
	 * @return void
	 */
	static function listmsg($nick)
	{
		$user = user::getuser($nick);
		$message = new message($user->id);
		
		$count = $message->get_new(50);
		
		if ($count instanceof error)
			return self::$client->msg($nick, "An internal error has been logged. Please try again later.");
			
		elseif ($count == 0)
			return self::$client->msg($nick, "You have no new messages. Login to the site for all messages.");
		
		/* Relay the list of messages */	
		else 
		{
			self::$client->msg($nick,"New Mail Messages");
			
			while ($each = $message->each())
			{
				/* trim the subject */
				if (strlen($each['subject']) > 25)
					$subject = substr($each['subject'], 0, 25). '...';
				
				else 
					$subject = $each['subject'];
					
				self::$client->msg($nick,"\0033#".$each['id']."\003 from \0037".$each['username']."\003 - ".$subject);
			}
			
			self::$client->msg($nick, "To read an entry use the READ command.");
		}
	}
	
	/**
	 * Command not known response
	 *
	 * @param string $nick
	 * @param string $command
	 * @return null
	 */
	static function unknowncmd($nick,$command)
	{	
		return self::$client->msg($nick, "Unknown command: \2{$command}\2; Need help? /msg ".memoserv::$client->nick." HELP");
	}
}

?>