<?php

/**
 * BuddyServ IRC Services
 * 
 * Message object handler. Used to retrieve and create IRC ->
 * website mail messages.
 * 
 * @name Message
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Core
 * @version $Id: Class.message.php 318 2009-11-14 04:17:22Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class message
{
	protected $list = array();
	protected $added;
	protected $read = false;
	protected $id;
	protected $sender_name;
	protected $owner = 0;
	protected $folder = 'Inbox';
	protected $subject;
	protected $body;
	private $db;
	private $irc;
	
	/**
	 * Constructs the message object
	 *
	 * * If subject is not given, we assume we will be accessing the database
	 * @param integer $owner
	 * @param string $subject
	 */
	public function __construct($owner, $subject=false)
	{
		global $db, $irc;
		
		$this->db = &$db;
		$this->irc = &$irc;
		
		/* simple sanity checks */
		if (!intval($owner))
			new error(false, "Failed creating Message Object. Non-Integer owner in __construct()");
			
		else 
			$this->owner = $owner;
			
		/* check if we have a valid subject */
		if ($subject)
		{
			$this->subject = $subject;
		}
	}
	
	public function __get($what)
	{
		if (isset($this->{$what}))
			return $this->{$what};
			
		return null;
	}
	
	/**
	 * Retrieves list of new mail messages from the database
	 *
	 * @param integer $max
	 * @return integer/error
	 */
	public function get_new($max)
	{
		/* sanity */
		if ($this->owner == 0)
			return new error(false, "No owner specified in Message->get_new()");
			
		/* check for greater than normal numbers */
		if ((!is_integer($max)) or ($max < 5) or ($max > 50))
		{
			/* default it to 5 */
			$max = 5;
		}
		
		$query = $this->db->query("SELECT m.*, u.username FROM `messages` m LEFT JOIN `users` u ON u.id=m.sender WHERE m.receiver = '".$this->owner."' AND m.unread = 'yes' AND m.saved = 'no' AND m.location = 1 LIMIT ".$max);
		
		/* No results? return 0 */
		if ($this->db->num_rows($query) == 0)
			return 0;
		
		$this->list = array();
				
		while ($array = $this->db->fetch_array($query))
		{
			if (!$array['username'])
				$array['username'] = 'System';
				
			$message_id = $array['id'];
			$this->list[$message_id] = $array;
		}
		
		return count($this->list);
	}
	
	/**
	 * Set the current message status as 'saved'
	 * 
	 * @return bool
	 */
	public function save()
	{
		if (!$this->id)
		{
			new error(false,"Message.save() on invalid message object.");
			return false;
		}
		
		$this->db->query("UPDATE `messages` SET `unread` = 'no', `saved` = 'yes' WHERE `id` = '".$this->id."' LIMIT 1");
		return true;
	}
	
	
	/**
	 * Add a new message to the database
	 * 
	 * @param int $sender
	 * @param string $subject
	 * @param string $body
	 * @return bool
	 */
	public function send($sender, $subject, $body)
	{
		$this->db->query("INSERT INTO `messages` (`sender`, `receiver`, `added`, `subject`, `msg`, `location`) 
		VALUES ('".$sender."', '".$this->owner."', FROM_UNIXTIME('".current_time()."'), '".$subject."', '".$body."', 1)");
		
		return true;
	}
	
	/**
	 * Cycles the current message list and outputs one by one
	 *
	 * @return unknown
	 */
	public function each()
	{
		/* nothing left in the message list? */
		if (count($this->list) == 0)
			return false;
			
		$next = array_shift($this->list);
		
		return $next;
	}
	
	/**
	 * Delete the current message from the database.
	 * 
	 * @return bool
	 */
	public function delete()
	{
		if (!$this->id)
		{
			new error(false, "Message.delete() called in unvalid message object.");
			return false;
		}
		
		$this->db->query("DELETE FROM `messages` WHERE `id` = '".$this->id."' LIMIT 1");
		return true;
	}
	
	/**
	 * Fetch a message with the given id from the server
	 * and stores it in the current object.
	 * 
	 * @param int $id
	 * @return bool
	 */
	public function fetch($id)
	{
		/* sanity */
		if (!intval($id))
			return new error(false, "Message->fetch called with non-integer ID");
			
		$query = $this->db->query('SELECT m.*,u.username FROM `messages` m LEFT JOIN `users` u ON u.id=m.sender WHERE m.id = '.$id.' LIMIT 1');
		
		if ($this->db->num_rows($query) == 0)
			return false;
			
		$array = $this->db->fetch_array($query);
		
		$this->read = ($array['unread'] == 'yes' ? true : false);
		$this->owner = $array['receiver'];
		$this->subject = $array['subject'];
		$this->body = $array['msg'];
		$this->sender_name = ($array['username'] ? $array['username'] : 'System');
		$this->added = $array['added'];
		$this->id = $id;
		
		return true;
	}	
}
			
		
?>	