<?php

/**
 * BuddyServ IRC Services
 * 
 * Monitor IRC pseudo client handler. Manages
 * extra service functions, and graphing commands.
 * 
 * @name Monitor
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.monitor.php 330 2009-11-20 06:02:20Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class monitor
{
	static $client;
	static $automated =false;
	static $last_read =0;
	static $failures  =0;
	static $verbose   =true;
	static $repeat = true;
	static $auto_matches=array();
	static $enabled=true;
	static $last_rrd=0;
	static $rrd_time='-604800';
	
	/**
	 * Initialize the current service client
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	static function initialize($nick, $username, $host, $name)
	{
		self::$last_read = current_time();
		self::$client = new serviceclient();
		self::$client->initialize($nick, $username, $host, $name);
		self::load_matches();
	}
	
	/**
	 * magic get.
	 * 
	 * @param string $what
	 * @return mixed
	 */
	function __get($what)
	{
		if (self::$client->$what)
			return self::$client->$what;
			
	}
	
	/**
	 * Introduce the client to the network.
	 * 
	 * @return void
	 */
	static function introduce()
	{
		self::$client->introduce();
	}
	
	/**
	 * Check if its time to update the rrd graph, and
	 * create an image for the most recent numbers.
	 * 
	 * @return void
	 */
	static function check_rrd()
	{
		global $me, $irc;
		
		if ($me->use_rrd)
		{
			/* check if its time to run the update */
			if ((self::$last_rrd == 0) or ((current_time() - self::$last_rrd) > 300))
			{
				$count = user::count_online();
				$guest = user::count_guests();

				exec('rrdtool update '.$me->rrd_file.' N:'.$count.':'.$guest);
				
				/* Graph the current irc users first */
				rrd::$title = 'IRC Graphs';
				rrd::$vertical = 'User Count';
				
				rrd::update(array($count, $guest));
				rrd::graph('rrd/graphs/hour.png', '-3600');
				rrd::graph('rrd/graphs/day.png', '-86400');
				rrd::graph('rrd/graphs/week.png', '-604800');
				rrd::graph('rrd/graphs/month.png', '-2419200');
				rrd::graph('rrd/graphs/year.png', '-31536000');
				
				/* Generate Lines Graph */
				rrd::$rrdfile = 'rrd/irclines.rrd';
				rrd::$title = 'IRC Send/Recv';
				rrd::$vertical = 'Send/Receive Lines';
				
				rrd::update(array(($irc->recvLines + $irc->sendLines), $irc->recvLines, $irc->sendLines));
				
				$irc->recvLines = 0;
				$irc->sendLines = 0;
				
				rrd::graphLines('rrd/graphs/hourlines.png', '-3600');
				rrd::graphLines('rrd/graphs/daylines.png', '-86400');
				rrd::graphLines('rrd/graphs/weeklines.png', '-604800');
				rrd::graphLines('rrd/graphs/monthlines.png', '-2419200');
				rrd::graphLines('rrd/graphs/yearlines.png', '-31536000');

				
				//exec($query);
				
				self::$last_rrd = current_time();
			}
		}
	}
			
			
	/**
	 * Check for intermessage from seedbox clients.
	 * 
	 * @return void
	 */
	static function check_active()
	{
		global $db, $me;
		
		if (!self::$enabled)
			return;
			
		$currtime = current_time();
		
		if ((current_time() - self::$last_read) < 3)
			return;
		/* check the database for message before our saved time.
		 * we are forcing the time to ensure we only delete message
		 * that have been read if something happens between the script.
		 */
		$query = $db->query("SELECT * FROM `buddyserv_interm` WHERE `added` < ".$currtime." ORDER BY `added` ASC");
		
		if ($db->num_rows($query) == 0)
		{
			self::$failures++;
			
			/* ping time is 190sec, so, 2.5min, server has 7.5mins to post a message
			 * to sql before services starts warning. This will happen every 7.5mins */
			
			if (self::$failures % 150 == 0)
			{
				self::$repeat = false;
				self::$client->msg($me->uploader_channel, "\0034Warning:\003 #".self::$failures.' - No response from seedbox in aprox '.(2*self::$failures).'mins next repeat 15mins');
			}
			
			elseif (self::$failures % 50 == 0)
			{
				if (!self::$repeat)
					return;
					
				self::$automated = false;
				self::$client->msg($me->uploader_channel, "\0034Warning:\003 #".self::$failures.' - No response from seedbox in aprox '.(1*self::$failures).'mins.');
			}
			
			return;
		}
		
		else 
		{
			$msg = array();
			
			if (!self::$automated)
				self::$client->msg($me->uploader_channel, "\0033Notice:\003 seedbox has responded after ".duration(self::$last_read,current_time()));
				
			self::$last_read = current_time();
			self::$automated = true;
			self::$failures = 0;
			
			while ($each = $db->fetch_array($query))
			{
				if ($each['message'] == 'PING')
					continue;
					
				$msg[] = "\00314[\003".$each['from']."\00314]\003 ".duration($each['added'], current_time()).' ago: '.$each['message'];
			}
			
			if (count($msg) > 0)
			{
				foreach ($msg as $line)
				{
					self::$client->msg($me->uploader_channel, $line);
				}
			}
			
			 $db->query("DELETE FROM `buddyserv_interm` WHERE `added` < ".$currtime);
			
		}
	}
		
	/**
	 * Called to parse any messages to monitor
	 *
	 * @param string $nick
	 * @param string $arguments
	 * @return void
	 */
	static function parser($nick, $channel, $arguments)
	{
		if (!strcasecmp($arguments[0], '!seed'))
			return self::seed($nick, $channel, $arguments[1]);
			
		if (!strcasecmp($arguments[0], '!update'))
			return self::update($nick, $channel);
			
		if (!strcasecmp($arguments[0], '!disable'))
			return self::onoff($nick, $channel, false);
			
		if (!strcasecmp($arguments[0], '!enable'))
			return self::onoff($nick, $channel, false);
			
	//	if (stristr($arguments[0], '!stop'))
		//	return self::haltseed($nick, $channel, $arguments[1]);
	
	//	if (stristr($arguments[0], '!announce'))
		//	return self::announce($nick, $channel);
			
	//	if (stristr($arguments[0], '!quiet'))
		//	return self::quiet($nick, $channel);
			
		if (!strcasecmp($arguments[0], '!match'))
			return self::match($nick, $channel, $arguments);
			
	//	if (stristr($arguments[0], '!status'))
		//	return self::status($nick, $channel, $arguments);
			
		if (!strcasecmp($arguments[0], 'HELP'))
		{
			array_shift($arguments);
			
			if (count($arguments) == 0)
				return self::$client->help($nick);
			
			return self::$client->help($nick, $arguments);
		}
	}
	
	/**
	 * Enable or disable autoseed functions
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $enabled
	 * @return bool
	 */
	static function onoff($nick, $channel, $enabled)
	{
		$user = user::getuser($nick);
		
		if ((!$user->identified()) or ($user->level < SITE_UPLOADER))
			return false;
			
		if ($enabled)
			self::$enabled = true;
		
		else 
			self::$enabled = false;
			
			self::$client->msg($channel, "AutoSeed functions have been ".($enabled ? 'enabled' : 'disabled'));
	}
			
	/**
	 * Check if an entry is already in the
	 * autoseed table.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param array $arguments
	 * @return bool
	 */
	static function match($nick, $channel, $arguments)
	{
		global $db;
		
		$user = user::getuser($nick);
		
		if ((!$user->identified()) or ($user->level < SITE_UPLOADER))
			return false;
			
		if (!strcasecmp($arguments[1], 'add'))
		{
			if (self::match_exists($arguments[2]))
				return self::$client->msg($channel, "Sorry, '".$arguments[2]."' already has a match.");
				
				$db->query("INSERT INTO `buddyserv_autoseed` (`name`, `added`, `hits`) VALUES ('".mysql_real_escape_string($arguments[2])."', ".
					"'".current_time()."', 0)");
					
				self::load_matches();
				
				return self::$client->msg($channel, "\0033Added ".$arguments[2]." to autoseed list.\003");
		}
			 
			
	}
	
	/**
	 * Force updates
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @return mixed
	 */
	static function update($nick, $channel)
	{
		$user = user::getuser($nick);
		
		
		if ((!$user->identified()) or ($user->level < SITE_UPLOADER))
			return false;
		
		
		self::check_active();
	}
	
	/**
	 * Add a release to the seed table
	 * for the seedboxes next run
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $file
	 * @return void
	 */
	static function seed($nick, $channel, $file)
	{
		global $db, $me;
		
		if (!self::exists($file))
			return self::$client->msg($me->uploader_channel, "\0034Error\003: Unknown release: ".$file);
			
			$filesafe = mysql_real_escape_string($file);
			
			$query = $db->query("SELECT * FROM `buddyserv_releases` WHERE `name` = '".$filesafe."'");
			
			$array = $db->fetch_array($query);
			
			if ($array['start'] == 0)
			{
				 self::$client->msg($me->uploader_channel, "\0033Queued\003: ".$file);
				 $db->query("UPDATE `buddyserv_releases` SET `start` = 1 WHERE `id` = ".$array['id']." LIMIT 1");
			}
			
			elseif ($array['start'] == 1)
				return self::$client->msg($me->uploader_channel, "\0037Queue\003: already in queue: ".$file);
				
			elseif ($array['start'] == 2)
				return self::$client->msg($me->uploader_channel, "\0034Error\003: Already complete: ".$file);
	}
	
	/**
	 * Check if a match exists for auto-seeding
	 * 
	 * @param string $name
	 * @return bool
	 */
	static function match_exists($name)
	{
		if (count(self::$auto_matches) == 0)
			return false;
		
		foreach (self::$auto_matches as $preg => $array)
		{
			if (preg_match('/^'.$preg.'$/', $name))
				return true;
		}
		
		return false;
	}
	
	/**
	 * Load a list of all the matches we should attempt
	 * to autoseed without being told to.
	 * 
	 * @return void
	 */
	static function load_matches()
	{
		global $db;
		
		$query = $db->query("SELECT * FROM `buddyserv_autoseed`");
		
		if ($db->num_rows($query) > 0)
		{
			while ($each = $db->fetch_array($query))
			{
				$regex = str_replace('*', '.*', $each['name']);
				$regex = str_replace('?', '\S{1}', $regex);
				
				self::$auto_matches[$regex] = array(
					'name'	=> $each['name'],
					'added'	=> $each['added'],
					'hits'	=> $each['hits']);
					
			}
		}
	}
	
	/**
	 * Check if a file exists in the release table
	 * 
	 * @param string $name
	 * @return bool
	 */
	static function exists($name)
	{
		global $db;
		
		$name = mysql_real_escape_string($name);
		
		$query = $db->query("SELECT `id` FROM `buddyserv_releases` WHERE `name` = '".$name."'");
		
		if ($db->num_rows($query) > 0)
			return true;
			
		return false;
	}
	
	/**
	 * Add notifications from the prebot to the release
	 * database for future use.
	 * 
	 * @param string $root
	 * @param string $name
	 * @return void
	 */
	static function new_notification($root, $name)
	{
		global $me, $db;
		
		if (!self::$enabled)
			return;
			
		if (self::exists($name))
				return self::$client->msg($me->uploader_channel, "\0034Exists\003: ".$name." is a dupe.");
		
		if (self::match_exists($name))
		{
			$db->query("INSERT INTO `buddyserv_releases` (`name`, `added`, `category`, `start`) 
				VALUES ('".mysql_real_escape_string($name)."', '".current_time()."', '".mysql_real_escape_string($root)."', 1)");
			
			return self::$client->msg($me->uploader_channel, "\0033Added\003: [\00310".$root."\003] ".$name." to automatic download queue.");
		}
		 
		else
		{
				$db->query("INSERT INTO `buddyserv_releases` (`name`, `added`, `category`, `start`) 
				VALUES ('".mysql_real_escape_string($name)."', '".current_time()."', '".mysql_real_escape_string($root)."', 0)");
		
			return self::$client->msg($me->uploader_channel, "\0037Ignored\003: [\00310".$root."\003] ".$name);
		}
	}
}

?>