<?php

/**
 * BuddyServ IRC Services
 * 
 * UserBot pseudo client used for generic site->irc
 * events and reporting.
 * 
 * @name UserBot
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.userbot.php 330 2009-11-20 06:02:20Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class userbot
{
	static $username;
	static $host;
	static $name;
	static $nick;
	static $help_list = array();
	static $client;
	static $changes;

	/**
	 * Initialize the current service client.
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	static function initialize($nick, $username, $host, $name)
	{
		self::$client = new serviceclient();
		self::$client->initialize($nick, $username, $host, $name);
	}
	
	/**
	 * magic getter
	 * 
	 * @param string $what
	 * @return mixed
	 */
	function __get($what)
	{
		if (self::$client->$what)
			return self::$client->$what;
			
	}
	
	/**
	 * Introduce the client to the network
	 * @return unknown_type
	 */
	static function introduce()
	{
		self::$client->introduce();
	}
	
	/**
	 * Parse commands form a channel
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param array $arguments
	 * @return void
	 */
	static function command_parser($nick, $channel, $arguments)
	{
		$checkuser = user::getuser($nick);
		
		if ($checkuser->forbid)
			return self::$client->notice($nick, "Access denied. Your user account is disabled.");
			
		if (!$checkuser->identified())
			return self::$client->notice($nick, "Please identify to NickServ before using me.");
			
		if (!strcasecmp($arguments[0], '-user'))
			return self::cmd_user($nick, $channel, $arguments);
	
		if (!strcasecmp($arguments[0], '-next'))
			return self::cmd_tvrage_next($nick, $channel, $arguments);
			
		if (!strcasecmp($arguments[0], '-info'))
			return nickserv::info($nick, $arguments[1]); 
			
		if (!strcasecmp($arguments[0], '-connectable'))
			return self::connectable($nick, $channel, $arguments[1]); 
			
		if (!strcasecmp($arguments[0], '-idle'))
			return self::cmd_idle($nick, $channel, $arguments[1]);
			
		if (!strcasecmp($arguments[0], '-stats'))
			return self::cmd_stats($nick, $arguments[1]);
			
		if (!strcasecmp($arguments[0], '-changes'))
			return self::changes($nick);
			
		if (!strcasecmp($arguments[0], '-syslog'))
			return self::cmd_syslog($nick, $channel, $arguments[1]);
			
		if (!strcasecmp($arguments[0], '-help'))
			$arguments[0] = 'HELP';
		
		if (!strcasecmp($arguments[0], 'HELP'))
		{
			array_shift($arguments);
			
			if (count($arguments) == 0)
				return self::$client->help($nick);
			
			return self::$client->help($nick, $arguments);
		}
	}
	
	/**
	 * Turns sysop logging on or off
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param on|off $switch
	 * @return mixed
	 */
	static function cmd_syslog($nick, $channel, $switch)
	{
		global $me;
		
		$user = user::getuser($nick);
		
		if (!$user->identified() or ($user->level < SITE_SYSOP))
			return false;
			
		if ($channel != $me->sysop_channel)
			return false;
			
		if ($switch == "on")
		{
			$me->syslog_verbose = true;
			return self::$client->msg($channel,"\0033Relaying syslog events.\003");
		}
		
		else 
		{
			$me->syslog_verbose = false;
			return self::$client->msg($channel, "\0034Syslog relaying halted.\003");
		}
			
	}
		
	/**
	 * Outputs changelog file
	 * 
	 * @param string $nick
	 * @return void
	 */
	static function changes($nick)
	{
		$user = user::getuser($nick);
		
		if ((!$user->identified()))
			return;
			
		if (!is_array(self::$changes))
		{
			self::$changes = explode("\r\n", file_get_contents('changes.txt'));
		}
		
		foreach (self::$changes as $line)
		{
			self::$client->notice($nick, $line);
		}
	}
		
	static function cmd_stats($nick, $what)
	{
		switch(strtolower($what))
		{
			case 'help':
				self::$client->notice($nick, 'STATS <irc|site|help>');
			break;
			
			case 'irc':
				self::$client->notice($nick, "Soon to come");
			break;
			
			case 'site':
				self::$client->notice($nick, "Soon to come");
			break;
			
			default:
				self::$client->notice($nick, 'STATS <irc|site|help>');
			break;
		}
	}
	
	/**
	 * Performs user information command for site stats
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_user($nick, $channel, $arguments)
	{	
		if (isset($arguments[1]))
			$target = $arguments[1];
			
		else 
			$target = $nick;
			
		$infostring = self::user_stats_string($target);
		
		if (!$infostring)
			return self::$client->notice($nick, "{$target} not a registered member.");
			
		return self::$client->msg($channel, $infostring);
	}
	
	/**
	 * Is a person connectable?
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $who
	 * @return void
	 */
	static function connectable($nick, $channel, $who)
	{
		global $db;
		
		if (!$who)
			$who = $nick;
			
		$query = $db->query("SELECT users.username,peers.connectable as connectable FROM `users` LEFT JOIN peers ON peers.userid=users.id WHERE users.username = '".mysql_real_escape_string($who)."' LIMIT 1");
		
		if ($db->num_rows($query) == 0)
			return self::$client->notice($nick, "User {$who} is not a registered member.");
			
		$array = $db->fetch_array($query);
		
		if ($array['connectable'] == 'yes')
			return self::$client->msg($channel, "User {$who} is \0033CONNECTABLE\003");
			
		if ($array['connectable'] == 'no')
			return self::$client->msg($channel, "User {$who} is \0034UNCONNECTABLE\003");
			
		return self::$client->msg($channel, "I dont know if '{$who}' is connectable.");
	}
	
	/**
	 * Generates the user statistics string
	 * 
	 * @param string $who
	 * @return string
	 */
	static function user_stats_string($who)
	{
		global $db;
		
		$who = mysql_real_escape_string($who);
		
		$query = $db->query("SELECT `username`,`class`,`downloaded`,`uploaded`,`title` FROM `users` WHERE `username` = '".$who."' LIMIT 1");
		
		if ($db->num_rows($query) == 0)
			return false;
			
		$on_irc = user::getuser($who);
		
		if ($on_irc)
		{
			if ($on_irc->identified())
				$irc_status = 'Yes';
				
			else 
				$irc_status = 'No';
		}
		
		else 
			$irc_status = 'No';
			
		$array = $db->fetch_array($query);
		$level = $array['class'];
		
		if ($array['downloaded'] == 0)
			$ratio = 'inf';
		
		else 
			$ratio = sprintf("%.2f", $array['uploaded'] / $array['downloaded']);
			
		$alldown = convert_bytes($array['downloaded']);
		$allup = convert_bytes($array['uploaded']);
			
		$string = $array['username'].' - Class: ('.user::$levels[$level].") - Uploaded:\0033 ".$allup['size']."\003 ".$allup['byte']." - Downloaded:\0034 ".$alldown['size']."\003 ".$alldown['byte'].' - Ratio: ('.$ratio.') - OnIRC: ('.$irc_status.')';
		
		return $string;
	}
	
	/**
	 * Command for a users idle bonus this session.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $target
	 * @return void
	 */
	static function cmd_idle($nick, $channel, $target=false)
	{	
		global $me;
		
		$target = ($target ? $target : $nick);
		
		$time = idle::get_time($target);
		
		if ($time)
		{
			$mb_got = idle::get_points($target);
			//$time_spent = sprintf("%.2f", ((time() - $time) / 60 / 60));
		//	$mb_got = sprintf("%.0f MB", ($time_spent * IDLE_KARMAHOUR));
			
			self::$client->notice($nick, "{$target}, with [{$mb_got} MB] bonus recorded this session.");
			return true;
		}
		
		else 
		{
			self::$client->notice($nick, "{$target} not currently receiving idle bonuses. /join ".$me->support_channel." if you need assistance.");
			return true;
		}
	}
	
	/**
	 * Check for new torrents, users, reports, syslog, ect
	 * 
	 * @return void
	 */
	static function cmd_report_check()
	{
		global $db, $last_report_check, $me;
		
		/*
		
		$query = $db->query("SELECT reports.*,users.username FROM `reports` LEFT JOIN `users` ON users.id=reports.reported_by WHERE reports.added > FROM_UNIXTIME({$last_report_check})");
		
		if ($db->num_rows($query) > 0)
		{
			while ($each = $db->fetch_array($query))
			{
				//enum('User','Comment','Request_Comment','Offer_Comment','Request','Offer','Torrent','Hit_And_Run','Post')
				
				$username = false;
				$url = false;
				
				switch ($each['reporting_type'])
				{
					case 'User':
						$query = $db->query('SELECT `username` FROM `users` WHERE `id` = '.$each['reporting_what'].' LIMIT 1');
						$username = $db->fetch_array($query);
						$username = $username['username'];
						$url = $me->url.'/userdetails.php?id='.$each['reporting_what'];
						break;
					
					case 'Torrent':
						$url = $me->url.'/details.php?id='.$each['reporting_what'];
						break;
						
					case 'Post':
						$url = $me->url.'/forums.php?action=viewtopic&topicid='.$each['2nd_value'].'&page=last#'.$each['reporting_what'];
						break;
				}
							
				if (!$url)
					continue;
					
				$reason = (strlen($each['reason']) < 150 ? $each['reason'] : substr($each['reason'], 0, 147).'...[more]');
				
				self::$client->msg($me->staff_channel, "\0034".$each['reporting_type']." Report:\003 ".($username ? $username.' ' : '')."[".$each['added']."] by ".$each['username']." - \00312{$url}\003");
				self::$client->msg($me->staff_channel, "Reason: {$reason}");
			}
		}
		*/
		/* Check for new torrents */
		//$query = $db->query("SELECT `id`, `save_as`, `size`, UNIX_TIMESTAMP(`added`) as `added` FROM `torrents` WHERE `added` > FROM_UNIXTIME({$last_report_check})");
		$query = $db->query("SELECT `id`, `save_as`, `size`, UNIX_TIMESTAMP(`added`) as `added` FROM `torrents` WHERE `added` > {$last_report_check}");
		//self::$client->msg("#coders", "Checking for new torrents. (".time().")"); //Debug INfo
		
		if ($db->num_rows($query))
		{
		    while ($each = $db->fetch_array($query))
		    {
				$array = byte_convert($each['size']);
				$size = $array['size'];
				$byte = $array['byte'];
		
			//	self::$client->msg($me->user_channel, "\2Torrent:\2 (".$size.$byte.")\0037 ".$each['save_as']."\003 - \00312http://torrentmax.org/details.php?id=".$each['id']."\003");

				/* Pre doesnt exist this release 
				$pre_query = $db->query("SELECT *, UNIX_TIMESTAMP(timestamp) AS timestamp FROM pre WHERE name = '".$each['save_as']."'");
				

				if(mysql_num_rows($pre_query) > 0)
				{
				    $rls = mysql_fetch_assoc($pre_query);
				    $rls_time = $each['added'] - $rls['timestamp'];
				    $days = floor($rls_time / (60 * 60 * 24));
				    $remainder = $rls_time % (60 * 60 * 24);
				    $hours = floor($remainder / (60 * 60));
				    $remainder = $remainder % (60 * 60);
				    $minutes = floor($remainder / 60);
				    $seconds = $remainder % 60;
				    
				    $rls_time_ago = '';
				    
				    if($days > 0)
				        $rls_time_ago .= $days.' Days, ';
				        
				    if($hours > 0)
				        $rls_time_ago .= $hours.' Hours, ';
				        
				    if($minutes > 0)
				        $rls_time_ago .= $minutes.' Minutes, ';
				        
				    $rls_time_ago .= $seconds.' Seconds';
				    
				    self::$client->msg($me->user_channel, "\2Torrent:\2 (".$size.$byte.")\0037 ".$each['save_as']."\003 - \00312".$me->url."/details.php?id=".$each['id']."\003 [pred ".$rls_time_ago." ago]");
				} */

			//	else
			//	{
				    self::$client->msg($me->user_channel, "\2Torrent:\2 (".$size.$byte.")\0037 ".$each['save_as']."\003 - \00312".$me->url."/details.php?id=".$each['id']."\003");
			//	}	       
			}
		}
	
		/* End of new torrent notification */
		
		/* Begin New User notification */
		
	//	$query = $db->query("SELECT `id`,`email`,`added`,`ip`,`username` FROM `users` WHERE `added` > FROM_UNIXTIME({$last_report_check})");
		$query = $db->query("SELECT `id`,`email`,`added`,`ip`,`username` FROM `users` WHERE `added` > {$last_report_check}");
		if ($db->num_rows($query))
		{
			while ($each = $db->fetch_array($query))
			{
		//		print_r($each);
				$lines = array();
				/* check for double IP */
				$ip = $each['ip'];
				
				//	$invited = (($each['invitedby'] > 0) ? " \2Invited: \2\0033yes\003 " : ' ');
	
					$lines[] = "\2New User:\2 \0037".$each['username']."\003 \2Ip:\2 *.".substr($ip,(strlen($ip) - 6), strlen($ip))." \2Url:\2 \00312".$me->url."/userdetails.php?id=".$each['id']."\003";
			}
				
			foreach ($lines as $message)
			{
				self::$client->msg($me->staff_channel, $message);
			}
		}
		
		/* Begin syslog reporting */
		if ($me->syslog_verbose)
		{
			/*
			$query = $db->query("SELECT * FROM `infolog` WHERE `added` > FROM_UNIXTIME({$last_report_check})");
			
			if ($db->num_rows($query))
			{
				while ($each = $db->fetch_array($query))
				{
					$message = $each['txt'];
					$message = preg_replace('#<a[^>]+href=[^>]*>(.*?)</a>#is', "$1", $message);
					
					
					self::$client->msg($me->sysop_channel, "[\0035syslog\003] ".$message);
				}
			} */
		}
			
	}
	
	
	/**
	 * Retrieves next showtime (coded by ozatomic or TheFool)
	 * cant remember which.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_tvrage_next($nick, $channel, $arguments)
	{	    
	    /* minor change to allow spaces in requests.
	     * automatically converts them to _ --lynk */
	    if (isset($arguments[1]))
	    {
	    	if (count($arguments) > 2)
	    	{
	    		array_shift($arguments);
	    		
	    		$show = implode('_', $arguments);
	    	}
	    	
	    	else 
	    		$show = $arguments[1];
	    		
		$fh = @fopen('http://www.tvrage.com/quickinfo.php?show='.$show, 'r');
		
		if($fh)
		{
		    $i = 0;
		    $lines = array();
		    
		    while (!feof($fh))
		    {
		        $i++;
				$buffer = fgets($fh, 4096);
				$buffer = explode('@', str_replace(array('^',"\n"), array(' ',''), $buffer));
				
			switch($buffer[0])
			{
		            case 'No Show Results Were Found For':
			        return self::$client->notice($nick, "{$show} Was not found");
				break;
			    case 'Show Name':
			        $lines[] = "Show:\0037 ".$buffer[1]."\003";
				break;
			    case 'Show URL':
			        $lines[] = "URL:\00312 ".$buffer[1]."\003";
				break;
			    case 'Latest Episode':
			        $lines[] = "Latest Ep:\0039 ".$buffer[1]."\003";
				break;
			    case 'Next Episode':
			        $lines[] = "Next Ep:\0037 ".$buffer[1]."\003";
				break;
			    case 'Status':
			        $lines[] = 'Status: '.$buffer[1];
				break;
			    case 'Airtime':
			        $lines[] = "Airtime:\0034 ".$buffer[1]."\003";
				break;
			   
			    case 'Premiered':
			    case 'RFC3339':
			    case 'Classification':
			    case 'Genres':
			    case 'Network':
			        default:
				break;
		       }
		   }
		   $string = implode(' :: ', $lines);
		   fclose($fh);
		   
		   if(!empty($string))
		       return self::$client->msg($channel, $string);
	       } else
	       {
	           return self::$client->msg($channel, "Connection to TVRage could not be made.");
	       }
	   }
	   return self::$client->msg($channel, "Usage: -next <show_name>");
	}
}
	
